#ifndef STACK_HPP_INCLUDED
#define STACK_HPP_INCLUDED

template <class T> // elemtípus sablonja
class Stack // verem típus
{
    private:
        T* _values; // értékek mutatója
        int _top; // a tetőelem indexe
        int _size; // méret

        void resize(int newSize); // átméretezés megadott méretre

    public:
        enum Exceptions { STACK_EMPTY }; // kivételek felsorolási típusa

        Stack(); // konstruktor (már nem adunk meg méretet)
        Stack(const Stack<T>& other); // másoló konstruktor
        ~Stack(); // destruktor
        bool isEmpty() const; // üres-e a verem
        void push(T value); // elem behelyezése
        T pop(); // elem kivétele
        T top() const; // tetőelem lekérdezése
        void clear(); // elemek törlése
        int size() const; // méret lekérdezése

        Stack<T>& operator=(const Stack<T>& other); // értékadás operátor
};

template <class T>
void Stack<T>::resize(int newSize)
{
    T *temp = new T[newSize]; // új tömb lefoglalása
    for (int i = 0; i < _top; i++)
    {
        temp[i] = _values[i]; // értékek átmásolása
    }
    delete[] _values; // régi tömb törlése
    _values = temp; // új tömb beállítása
    _size = newSize; // eltároljuk az új méretet
}

template <class T>
Stack<T>::Stack()
{
    _values = new T[100]; // tömb dinamikus létrehozása (megadunk egy alapértelmezett méretet)
    _size = 100; // méret eltárolása
    _top = 0; // kezdetben 0 elem van a veremben
}

template <class T>
Stack<T>::Stack(const Stack<T>& other) {
   _size = other._size; // átmásoljuk az egyszerű értékeket
   _top = other._top;

   _values = new T[_size]; // létrehozzuk a dinamikus tömböt

   for (int i = 0; i < _top; i++)
     _values[i] = other._values[i]; // átmásoljuk a hasznos értékeket
}


template <class T>
Stack<T>::~Stack()
{
    delete[] _values; // dinamikus tömb törlése
}

template <class T>
bool Stack<T>::isEmpty() const
{
    return _top == 0;
}

template <class T>
void Stack<T>::push(T value)
{
    if (_top == _size) // ha már nincs hely
        resize(2 * _size); // átméretezés a duplájára

    _values[_top] = value; // behelyezzük az elemet
    _top++; // növeljük az elemszámot
}

template <class T>
T Stack<T>::pop()
{
    if (_top > 0) // ha van elem
    {
        if (_size > 100 && _top < _size / 4) // ha már nagyon kicsi a verem
            resize(_size / 2); // átméretezzük a felére

        _top--; // csökkentjük az elemszámot
        return _values[_top]; // visszaadjuk a kivett elemet
    }
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
T Stack<T>::top() const
{
    if (_top > 0) // ha van elem
        return _values[_top - 1];
    else // különben kivételt dobunk
        throw STACK_EMPTY;
}

template <class T>
void Stack<T>::clear()
{
    _top = 0;
    resize(100);
}

template <class T>
int Stack<T>::size() const
{
    return _top;
}

template <class T>
Stack<T>& Stack<T>::operator=(const Stack<T>& other) {
   if (this == &other) // amennyiben a két memóriacím megegyezik
      return *this; // nem kell semmit csinálni

   delete[] _values; // eddigi dinamikus tömb törlése

   _size = other._size; // átmásoljuk az egyszerű értékeket
   _top = other._top;

   _values = new T[_size]; // létrehozzuk a dinamikus tömböt

   for (int i = 0; i < _top; i++)
     _values[i] = other._values[i]; // átmásoljuk a hasznos értékeket

   return *this;
}

#endif // STACK_HPP_INCLUDED
